/*
 * CarApplet.java
 */

import java.awt.*;
import java.util.*;

class Car {
    public double x,y;
    public double speed;
    public double acceleration;
    public double direction;
    public double friction;
    public int w,h;

    public Car (double x,double y,int w,int h){
        this.x = x;
        this.y = y;
        speed = 0;
        acceleration = 0;
        direction = 0;
        friction = 0.1;
        this.w = w;
        this.h = h;
    }

    public void move(){
        speed += acceleration;
        speed -= friction * speed;
        x += Math.cos(direction)*speed;
        y += Math.sin(direction)*speed;
        if(x > w)
            x = 0;
        if(x < 0)
            x = w;
        if(y > h)
            y = 0;
        if(y < 0)
            y = h;
    }

    public void draw(Graphics g){
        double[] xs = {-1,2,2,-1};
        double[] ys = {1,1,-1,-1};

        for(int i=0;i<xs.length;i++){
            xs[i]*=10;
            ys[i]*=10;
        }

        // rotate and translate. 
        int[] tx = new int[xs.length];
        int[] ty = new int[ys.length];
        // cos, -sin
        // sin, cos
        double sin = Math.sin(direction);
        double cos = Math.cos(direction);
        for(int i=0;i<xs.length;i++){
            tx[i] = (int)
                (x + (cos * xs[i] + -sin * ys[i]));
            ty[i] = (int)
                (y + (sin * xs[i] + cos * ys[i]));
            ty[i] = ty[i] * -1 + h;
        }
        g.setColor(Color.red);
        g.fillPolygon(tx,ty,tx.length);
        g.setColor(Color.black);
        g.drawPolygon(tx,ty,tx.length);
    }
}


public class CarApplet extends BufferedApplet {

    boolean[] keys;
    Car car;
    int vx,vy;

    /**
     * initialize things
     */
    public void init(){
        super.init();
        keys = new boolean[0x1001];
        car = new Car(bounds().width / 2, bounds().height / 2,bounds().width,bounds().height);
        vx=(int)(Math.random()*bounds().width);
        vy=(int)(Math.random()*bounds().height);
    }

    /**
     * function that gets called whenever something needs to be
     * rendered.
     */
    public void render(Graphics g) {
        // animate
        if(keys[1006] || keys['a']){ // left key
            car.direction += Math.PI/32;
        }

        if(keys[1007] || keys['d']){ // right key
            car.direction -= Math.PI/32;            
        }

        if(keys[1004] || keys['w']){ // up key
            car.acceleration+=0.1;
            car.acceleration = Math.min(3,car.acceleration);
        }else if(keys[1005] || keys['s']){ // down key
            car.acceleration-=0.1;
            car.acceleration = Math.max(-2,car.acceleration);
            
        } else {
            car.acceleration = 0;
        }

        car.move();

            // move victim away...
            if(destx < vx)
                vx+=3;
            if(destx > vx)
                vx-=3;
            if(desty < vy)
                vy+=3;
            if(desty > vy)
                vy-=3;
            
        // victim motion.
        if(vx > bounds().width)
            vx -= bounds().width;
        if(vx < 0)
            vx += bounds().width;
        if(vy > bounds().height)
            vy -= bounds().height;
        if(vy < 0)
            vy += bounds().height;
        
        g.setColor(Color.lightGray);
        g.fillRect(0, 0, bounds().width, bounds().height);

        car.draw(g);

        g.setColor(Color.blue);
        g.fillRect(vx-2,(vy-2) * -1 + bounds().height,5,5);
        g.setColor(Color.black);
        g.drawRect(vx-2,(vy-2) * -1 + bounds().height,5,5);
    }

    public boolean keyDown(Event evt,int key){
        //System.out.println("keyDown: "+key);
        if(key < 0x1000)
            keys[key] = true;
        return true;
    }

    public boolean keyUp(Event evt,int key){
        //System.out.println("keyUp: "+key);
        if(key < 0x1000)
            keys[key] = false;
        return true;
    }
}

